const Q = require('q');
const _ = require('underscore');
const to_boolean = require('../../../convert/to_boolean');
const __exec = require('../../__exec');

/**
 * NL类库
 *
 * @module nllib
 */

/**
 * 执行命令，在stdout输出命令的执行结果
 *
 * @class nllib/src/child_process/exec_cmd
 * @static
 *
 * @param {object} pParameter
 *
 * @param {string} pParameter.command 命令字符串
 * @param {boolean} pParameter.enable_log 是否输出日志，默认值为true
 * @param {string} pParameter.options options 传入到child_process.exec的options
 * @param {function} pParameter.stdout_data_event_handler stdout的data事件处理方法
 * @param {function} pParameter.stderr_data_event_handler stderr的data事件处理方法
 * @param {function} pParameter.exit_event_handler exit事件处理方法
 * @param {function} pParameter.close_event_handler close事件处理方法
 * @return {object} promise，当且仅当exit事件和close事件都执行了才解决
 *
 * @demo src/child_process/exec_cmd/1.0.2/demo/index.demo.js
 */
const exec_cmd = function (pParameter) {
    if (!pParameter) pParameter = {};
    const command = pParameter.command;
    const enable_log = to_boolean(pParameter.enable_log, true);
    const options = pParameter.options;
    const stdout_data_event_handler = pParameter.stdout_data_event_handler;
    const stderr_data_event_handler = pParameter.stderr_data_event_handler;
    const exit_event_handler = pParameter.exit_event_handler;
    const close_event_handler = pParameter.close_event_handler;

    if (!command) {
        const msg = 'nllib/src/child_process/exec_cmd parameter error, code=001';
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }

    const defer_exit = Q.defer();
    const defer_close = Q.defer();

    const returnValueExec = __exec('exec', command, options);
    returnValueExec.stdout.on(`data`, function (chunk) {
        if (_.isFunction(stdout_data_event_handler)) {
            stdout_data_event_handler(chunk);
            return;
        }
        const textChunk = chunk.toString(`utf8`);
        if (enable_log) console.log(textChunk);
    });
    returnValueExec.stderr.on(`data`, function (chunk) {
        if (_.isFunction(stderr_data_event_handler)) {
            stderr_data_event_handler(chunk);
            return;
        }
        const textChunk = chunk.toString(`utf8`);
        if (enable_log) console.log(textChunk);
    });

    returnValueExec.on(`exit`, function (code) {
        if (_.isFunction(exit_event_handler)) {
            exit_event_handler(code);
            if (code === 0) {
                defer_exit.resolve({code});
            } else {
                defer_exit.reject({code});
            }
            return;
        }
        if (enable_log) console.log(`nllib/src/child_process/exec_cmd:`, `exit code`, `${code}`);
        if (code === 0) {
            defer_exit.resolve({code});
        } else {
            defer_exit.reject({code});
        }
    });
    returnValueExec.on(`close`, function (code) {
        if (_.isFunction(close_event_handler)) {
            close_event_handler(code);
            if (code === 0) {
                defer_close.resolve({code});
            } else {
                defer_close.reject({code});
            }
            return;
        }
        if (enable_log) console.log(`nllib/src/child_process/exec_cmd:`, `close code`, `${code}`);
        if (code === 0) {
            defer_close.resolve({code});
        } else {
            defer_close.reject({code});
        }
    });

    return Q.all([defer_exit.promise, defer_close.promise,]);
};
module.exports = exec_cmd;

// 参考：
// https://stackoverflow.com/questions/37522010/difference-between-childprocess-close-exit-events
// https://stackoverflow.com/questions/18894433/nodejs-child-process-working-directory
