// const path = require(`path`);
const fs = require(`fs-extra`);
// const extend = require(`extend`);
const Q = require(`q`);
const _ = require(`underscore`);
// const moment = require(`moment`);
const XLSX = require('xlsx');

const to_boolean = require(`../../../convert/to_boolean`);

/**
 * @module nllib
 */

/**
 * 读取excel文件所有工作表数据
 * <br>依赖模块：XLSX
 *
 * @class nllib/src/excel-helper/base_read_excel_file
 * @static
 * @param {object} pParameter
 * @param {string} pParameter.path_file_excel excel文件路径
 * @param {string} pParameter.range 要读取数据的range，例如【D4:E999】，默认值为undefined
 * @param {string} pParameter.is_obtained_base_worksheet_object 返回的entity_list中，是否包含原始worksheet对象，默认值为否
 *
 * @return {object} excel文件所有工作表数据<br />
 * 数据格式如下：<br />
 * const o = {};<br />
 * o.sheet_name_list = ['sheet1', 'sheet2'];<br />
 * o.sheet_data_entity_list = [<br />
 * {sheet_name:`sheet1`, index:0, record_count:0, entity_list:[]},<br />
 * {sheet_name:`sheet2`, index:0, record_count:0, entity_list:[]},<br />
 * ];<br />
 * o.sheet_data_object = {<br />
 * 'sheet1':[], <br />
 * 'sheet2':[]<br />
 * };<br />
 *
 * @demo src/excel-helper/base_read_excel_file/1.0.0/demo/index.demo.js
 */
const base_read_excel_file = function (pParameter) {
    if (!pParameter) pParameter = {};
    const path_file_excel = pParameter.path_file_excel;
    const range = pParameter.range;
    const is_obtained_base_worksheet_object = to_boolean(pParameter.is_obtained_base_worksheet_object, false);

    if (!fs.existsSync(path_file_excel)) {
        const msg = `base_read_excel_file parameter error, code=001`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }

    const aWorkBook = XLSX.readFile(path_file_excel, {});
    const sheet_name_list = aWorkBook.SheetNames;
    if (sheet_name_list.length <= 0) {
        const msg = `base_read_excel_file parameter error, code=002`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }

    const sheet_data_entity_list = _.chain(sheet_name_list)
        .map(function (sheet_name, index) {
            const aWorkSheet = aWorkBook.Sheets[sheet_name];
            const entity_list = XLSX.utils.sheet_to_json(aWorkSheet, {
                // header: 1,
                // header: `A`,
                range: range,
                defval: ``,
            });
            const record_count = entity_list.length;

            const o = {
                sheet_name, index, record_count, entity_list, // aWorkSheet,
            };
            if (is_obtained_base_worksheet_object) {
                o.aWorkSheet = aWorkSheet;
            }
            return o;
        })
        .value();

    const sheet_data_object = {};
    _.each(sheet_data_entity_list, function (sheet_data_entity, index) {
        const sheet_name = sheet_data_entity.sheet_name;
        const entity_list = sheet_data_entity.entity_list;
        sheet_data_object[sheet_name] = entity_list;
    });

    return {
        sheet_name_list, sheet_data_entity_list, sheet_data_object,
    };
};

module.exports = base_read_excel_file;
