/**
 * NL类库
 *
 * @module nllib
 */

/**
 * <div>wrap缓存</div>
 * <div>实现功能：在同一cache_key下，多次调用，只调用一次请求远程数据的方法</div>
 *
 * @class nllib/src/promise-helper/wrap_cache
 * @static
 *
 * @param {object} pParameter
 * @param {boolean} pParameter.clear_cache 清空缓存，默认值为false
 * @param {string} pParameter.cache_key 缓存key，默认值为fn.toString()
 * @param {function} pParameter.fn 请求远程数据的方法，必填，返回值必须为promise
 * @param {function} pParameter.get_parameter_for_fn 返回一个promise，该promise.then中的data，将传入到fn中
 * @return {object} promise，
 *
 * @demo src/promise-helper/wrap_cache/1.0.1/demo/default.demo.js
 */

const Q = require(`q`);
const _ = require(`underscore`);

let cache_list = [];

const wrap_cache = function (pParameter) {
    if (!pParameter) {
        pParameter = {};
    }

    const clear_cache = pParameter.clear_cache || false;
    const cache_key = pParameter.cache_key;
    const fn = pParameter.fn || null;
    const get_parameter_for_fn = pParameter.get_parameter_for_fn || null;

    if (!_.isFunction(fn)) {
        const msg = 'wrap_cache parameter error, code=001';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    let key;
    if (!cache_key) {
        key = fn.toString();
    } else {
        key = cache_key;
    }

    if (clear_cache) {
        cache_list = _.filter(cache_list, function (m) {
            return m.key !== key;
        });
    }

    const query = _.find(cache_list, function (m) {
        return m.key === key;
    });

    if (query) {
        const promise = query.value;
        return promise;
    }

    let promise = Q.when();

    if (_.isFunction(get_parameter_for_fn)) {
        promise = promise.then(function () {
            return get_parameter_for_fn();
        });
    }

    promise = promise.then(function (p) {
        return fn(p);
    }).catch(function (ex) {
        if (ex) {
            const enable_log_error = !ex.disable_log_error;
            if (enable_log_error) {
                console.error(ex);
                console.trace();
                if (ex.stack) {
                    console.error(ex.stack);
                }
            }
        }
        return Q.reject(ex);
    })
    ;

    cache_list.push({
        key: key,
        value: promise
    });
    return promise;
};

module.exports = wrap_cache;
