//<editor-fold desc="node core modules">
// const path = require('path');
//</editor-fold>

//<editor-fold desc="npm modules">
// const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
const _ = require('underscore');
// const moment = require('moment');
// const URI = require('urijs');
const ALY = require('@ningkyolei/aliyun-sdk');
//</editor-fold>

const get_uuid = require(`../../../../../common/get_uuid`);

/**
 * NL类库
 *
 * @module NL
 */

/**
 * 阿里云OpenSearch客户端
 * <div>
 * <span class="tag margin-right">不支持浏览器环境</span>
 * <span class="tag margin-right">支持node环境</span>
 * </div>
 * <br>官方文档：https://help.aliyun.com/document_detail/29140.html?spm=a2c4g.11186623.6.611.5a066d292jSgim
 *
 * @class NL.systemDocking.aliyun.openSearch.backend.client
 * @constructor
 *
 * @param {object} pInitParameter
 * @param {string} pInitParameter.accessKeyId 阿里云的accessKeyId
 * @param {string} pInitParameter.accessKeySecret 阿里云的accessKeySecret
 * @param {string} pInitParameter.endPoint 公网API域名
 * @param {string} pInitParameter.appName 应用名称
 *
 * @return {object} NL.systemDocking.aliyun.openSearch.backend.client实例
 *
 */
module.exports = function (pInitParameter) {
    const __this = this;

    const defaultInitParameter = {};
    defaultInitParameter.accessKeyId = ``;
    defaultInitParameter.accessKeySecret = ``;
    defaultInitParameter.endPoint = ``;
    defaultInitParameter.appName = ``;

    const initParameter = extend(true, {}, defaultInitParameter, pInitParameter);

    const accessKeyId = initParameter.accessKeyId;
    const accessKeySecret = initParameter.accessKeySecret;
    const endPoint = initParameter.endPoint;
    const appName = initParameter.appName;

    if (!accessKeyId) {
        const msg = 'NL.systemDocking.aliyun.openSearch.backend.client parameter error, code=001';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!accessKeySecret) {
        const msg = 'NL.systemDocking.aliyun.openSearch.backend.client parameter error, code=002';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!endPoint) {
        const msg = 'NL.systemDocking.aliyun.openSearch.backend.client parameter error, code=003';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!appName) {
        const msg = 'NL.systemDocking.aliyun.openSearch.backend.client parameter error, code=004';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    const __client = new ALY.OpenSearch({
        accessKeyId: accessKeyId,
        secretAccessKey: accessKeySecret,
        // 参考 opensearch 控制台，获取 endpoint
        endpoint: endPoint,
        // 这是 opensearch sdk 目前支持最新的 api 版本, 不需要修改
        apiVersion: '2015-01-01'
    });

    const uploadDoc = function (pParameter) {
        const __items = pParameter.items;
        const table_name = pParameter.table_name || `main`;

        const defer = Q.defer();

        const items = _.chain(__items)
            .map(function (m) {
                const o = extend(true, {}, m);
                const fields = o.fields;
                _.each(fields, function (property_value, property_name) {
                    // console.log(`property_name=${property_name}`);

                    let value;
                    if (_.isArray(property_value)) {
                        value = property_value;
                    } else {
                        value = property_value.toString();
                    }
                    // const value = property_value.toString();

                    //不转换成字符串，会报错！
                    fields[property_name] = value;
                });

                const cmd = o.cmd;
                switch (cmd) {
                    case `add`:
                        fields.id = get_uuid();
                        break;
                }
                return o;
            })
            .value()
        ;

        const options = {};
        options.app_name = appName;
        options.action = `push`;
        options.items = items;
        options.table_name = table_name;

        __client.uploadDoc(options, function (err, res) {
            // console.log(err, res);
            if (err) {
                defer.reject(err);
                return;
            }
            if (!res) {
                const msg = 'uploadDoc parameter error, code=501';
                console.error(msg);
                console.error(JSON.stringify(res, null, 4));
                defer.reject({msg: msg});
                return;
            }
            const status = res.status;
            if (status !== `OK`) {
                const msg = 'uploadDoc parameter error, code=502';
                console.error(msg);
                console.error(JSON.stringify(res, null, 4));
                defer.reject({msg: msg});
                return;
            }
            defer.resolve(res);
        });

        return defer.promise;
    };

    const search = function (pParameter) {
        const query = pParameter.query;
        const fetch_fields = pParameter.fetch_fields;

        const defer = Q.defer();

        const options = {};
        options.index_name = appName;
        options.query = query;
        options.fetch_fields = fetch_fields;

        __client.search(options, function (err, res) {
            // console.log(err, res);
            if (err) {
                defer.reject(err);
                return;
            }
            if (!res) {
                const msg = 'search parameter error, code=501';
                console.error(msg);
                console.error(JSON.stringify(res, null, 4));
                defer.reject({msg: msg});
                return;
            }
            const status = res.status;
            if (status !== `OK`) {
                const msg = 'search parameter error, code=502';
                console.error(msg);
                console.error(JSON.stringify(res, null, 4));
                defer.reject({msg: msg});
                return;
            }

            // console.log(JSON.stringify(res, null, 4));
            defer.resolve(res);
        });

        return defer.promise;
    };

    __this.__client = __client;

    /**
     * uploadDoc
     * <br>官方文档：https://help.aliyun.com/document_detail/57154.html?spm=a2c4g.11186623.2.18.1a2e7349fBeeyv
     * <div class="text-dot">
     * <br>官方文档上很重要的一句话：
     * <br>数据上传后请务必检查返回值，并对相关错误码进行重试（尤其是3007错误），否则会出现数据丢失情况。
     * <br>同时，数据处理是异步的，系统返回“OK”后只表示系统接收数据成功，数据处理过程的错误会在控制台错误信息中展示，请注意及时检查。
     * </div>
     * @method uploadDoc
     *
     * @param {object} pParameter
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/open-search/backend/client/demo/uploadDoc.demo.js
     */
    __this.uploadDoc = uploadDoc;

    /**
     * search
     * <br>官方文档：https://help.aliyun.com/document_detail/57155.html?spm=a2c4g.11186623.6.615.5a066d292jSgim
     * <div style="color:red;font-weight:bold;">
     * <br>特别注意：
     * <br>现象：openSearch，uploadDoc后马上search，有可能会出现不能查询到最新的记录的情况！
     * <br>原因：官方解释如下：数据处理是异步的，系统返回“OK”后只表示系统接收数据成功，数据处理过程的错误会在控制台错误信息中展示，请注意及时检查。
     * <br>参考：https://help.aliyun.com/document_detail/57154.html?spm=a2c4g.11186623.2.18.1a2e7349fBeeyv
     * </div>
     *
     * @method search
     *
     * @param {object} pParameter
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/open-search/backend/client/demo/search.demo.js
     */
    __this.search = search;
};