//<editor-fold desc="node core modules">
// const path = require('path');
//</editor-fold>

//<editor-fold desc="npm modules">
// const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
const _ = require('underscore');
// const moment = require('moment');
// const URI = require('urijs');
const OSS = require('ali-oss');
//</editor-fold>

/**
 * NL类库
 *
 * @module NL
 */

/**
 * 阿里云oss客户端
 * <div>
 * <span class="tag margin-right">不支持浏览器环境</span>
 * <span class="tag margin-right">支持node环境</span>
 * </div>
 * <br>官方文档：https://help.aliyun.com/document_detail/32070.html?spm=a2c4g.11186623.6.967.2bb710d5hWbtRn
 * <br>官方文档：https://github.com/ali-sdk/ali-oss
 *
 * @class NL.systemDocking.aliyun.oss.backend.client
 * @constructor
 *
 * @param {object} pInitParameter
 * @param {string} pInitParameter.accessKeyId 阿里云的accessKeyId
 * @param {string} pInitParameter.accessKeySecret 阿里云的accessKeySecret
 * @param {string} pInitParameter.region 地区，参考：https://github.com/ali-sdk/ali-oss
 * @param {string} pInitParameter.bucket bucket
 *
 * @return {object} NL.systemDocking.aliyun.oss.backend.client实例
 *
 */
module.exports = function (pInitParameter) {
    const __this = this;

    const defaultInitParameter = {};
    defaultInitParameter.accessKeyId = ``;
    defaultInitParameter.accessKeySecret = ``;
    defaultInitParameter.region = ``;
    defaultInitParameter.bucket = ``;

    const initParameter = extend(true, {}, defaultInitParameter, pInitParameter);

    const accessKeyId = initParameter.accessKeyId;
    const accessKeySecret = initParameter.accessKeySecret;
    const region = initParameter.region;
    const bucket = initParameter.bucket;

    if (!accessKeyId) {
        const msg = 'NL.systemDocking.aliyun.oss.backend.client parameter error, code=001';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!accessKeySecret) {
        const msg = 'NL.systemDocking.aliyun.oss.backend.client parameter error, code=002';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!region) {
        const msg = 'NL.systemDocking.aliyun.oss.backend.client parameter error, code=003';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!bucket) {
        const msg = 'NL.systemDocking.aliyun.oss.backend.client parameter error, code=004';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    const __client = new OSS({
        accessKeyId: accessKeyId,
        accessKeySecret: accessKeySecret,
        region: region,
        bucket: bucket,
    });

    const list = function (pParameter) {
        const func = __client.list;
        const context = __client;
        return func.apply(context, Array.prototype.slice.call(arguments));
    };

    const put = function (pParameter) {
        const func = __client.put;
        const context = __client;
        return func.apply(context, Array.prototype.slice.call(arguments));
    };

    const __delete = function (pParameter) {
        const func = __client.delete;
        const context = __client;
        return func.apply(context, Array.prototype.slice.call(arguments));
    };

    const deleteMulti = function (pParameter) {
        const func = __client.deleteMulti;
        const context = __client;
        return func.apply(context, Array.prototype.slice.call(arguments));
    };

    const multipartUpload = function (pParameter) {
        const __args = arguments;
        return Q.when()
            .then(function () {
                return __client.multipartUpload.apply(__client, Array.prototype.slice.call(__args));
            })
            .then(function (data) {
                if (!data) {
                    return Q.resolve(data);
                }

                if (data.url) {
                    return Q.resolve(data);
                }

                if (!data.res) {
                    return Q.resolve(data);
                }
                if (!data.res.requestUrls) {
                    return Q.resolve(data);
                }
                if (data.res.requestUrls.length <= 0) {
                    return Q.resolve(data);
                }

                const url_001 = decodeURI(data.res.requestUrls[0]);
                const url_002 = url_001.replace(/\?uploadId=.*/gi, ``);

                data.url = url_002;

                return Q.resolve(data);
            })
            .catch(function (ex) {
                console.error(ex);
                if (ex) {
                    if (ex.stack) {
                        console.error(ex.stack);
                    }
                }
                return Q.reject(ex);
            })
            ;
    };

    __this.__client = __client;

    /**
     * list
     * <br>sdk官方文档：https://github.com/ali-sdk/ali-oss#listquery-options
     * <br>官方文档：https://help.aliyun.com/document_detail/32074.html?spm=a2c4g.11186623.6.971.67646a38Nms6fk
     * @method list
     *
     * @param {object} pParameter
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/oss/backend/client/demo/list.demo.js
     */
    __this.list = list;

    /**
     * put
     * <br>sdk官方文档：https://github.com/ali-sdk/ali-oss#putname-file-options
     * <br>官方文档：https://help.aliyun.com/document_detail/31848.html?spm=a2c4g.11186623.6.597.4208b5ddLW1jLd
     * @method put
     *
     * @param {object} pParameter
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/oss/backend/client/demo/put.demo.js
     */
    __this.put = put;

    /**
     * delete
     * <br>sdk官方文档：https://github.com/ali-sdk/ali-oss#deletename-options
     * <br>官方文档：https://help.aliyun.com/document_detail/31862.html?spm=a2c4g.11186623.6.613.4208b5ddLW1jLd
     * @method delete
     *
     * @param {object} pParameter
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/oss/backend/client/demo/delete.demo.js
     */
    __this.delete = __delete;

    /**
     * deleteMulti
     * <br>sdk官方文档：https://github.com/ali-sdk/ali-oss#deletemultinames-options
     * <br>官方文档：https://help.aliyun.com/document_detail/31862.html?spm=a2c4g.11186623.6.613.4208b5ddLW1jLd
     * @method deleteMulti
     *
     * @param {object} pParameter
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/oss/backend/client/demo/deleteMulti.demo.js
     */
    __this.deleteMulti = deleteMulti;

    /**
     * multipartUpload
     * <br>sdk官方文档：https://github.com/ali-sdk/ali-oss#multipartuploadname-file-options
     * <br>官方文档：https://help.aliyun.com/document_detail/31850.html?spm=a2c4g.11186623.6.618.4cbf3d4cPnG0l3
     * @method multipartUpload
     *
     * @param {object} pParameter
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/oss/backend/client/demo/multipartUpload.demo.js
     */
    __this.multipartUpload = multipartUpload;

};