// const path = require(`path`);
// const fs = require(`fs-extra`);

const extend = require(`extend`);
const Q = require(`q`);
const _ = require(`underscore`);
const moment = require(`moment`);
// const URI = require(`urijs`);
const axios = require(`axios`);
const join = require(`../../../uri/join`);

/**
 * NL类库
 *
 * @module nllib
 */

/**
 * gitlab客户端
 *
 * <br><label>参考：</label>
 * <br><a style="color:red;" href="https://docs.gitlab.com/ee/api/" target="_blank">https://docs.gitlab.com/ee/api/</a>
 * <br><a style="color:red;" href="https://docs.gitlab.com/ee/api/api_resources.html" target="_blank">https://docs.gitlab.com/ee/api/api_resources.html</a>
 * <br><a style="color:red;" href="https://docs.gitlab.com/ee/api/commits.html" target="_blank">https://docs.gitlab.com/ee/api/commits.html</a>
 *
 * @class nllib/ssrc/system-docking/gitlab/client
 * @constructor
 *
 * @param {object} pInitParameter
 * @param {string} pInitParameter.gitlab_protocol gitlab的协议，默认值为http
 * @param {string} pInitParameter.gitlab_host gitlab的host
 * @param {string} pInitParameter.gitlab_api_version gitlab的api版本，默认值为v4
 * @param {string} pInitParameter.gitlab_private_token gitlab的某个用户的access_token
 *
 * @return {object} nllib/src/system-docking/aliyun/oss/frontend/client实例
 *
 */
const Client = function (pInitParameter) {
    const __this = this;

    const defaultInitParameter = {};
    defaultInitParameter.gitlab_protocol = `http`;
    defaultInitParameter.gitlab_host = ``;
    defaultInitParameter.gitlab_api_version = `v4`;
    defaultInitParameter.gitlab_private_token = ``;

    const initParameter = extend({}, defaultInitParameter, pInitParameter);

    const gitlab_protocol = initParameter.gitlab_protocol;
    const gitlab_host = initParameter.gitlab_host;
    const gitlab_api_version = initParameter.gitlab_api_version;
    const gitlab_private_token = initParameter.gitlab_private_token;

    if (!gitlab_host) {
        const msg = `Client init parameter error, code=001`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }

    if (!gitlab_private_token) {
        const msg = `Client init parameter error, code=002`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }

    const url_prefix = `${gitlab_protocol}://${gitlab_host}/api/${gitlab_api_version}`;

    const get_request_url = function (pParameter) {
        const defaultParameter = {};
        defaultParameter.url_suffix = ``;
        defaultParameter.gitlab_private_token = gitlab_private_token;
        defaultParameter.p = {};

        const parameter = extend({}, defaultParameter, pParameter);

        const url_suffix = parameter.url_suffix;
        const __gitlab_private_token = parameter.gitlab_private_token;
        const p = parameter.p;

        if (!url_suffix) {
            const msg = `get_request_url parameter error, code=001`;
            console.error(msg);
            console.trace();
            return Q.reject({msg: msg});
        }
        if (!__gitlab_private_token) {
            const msg = `get_request_url parameter error, code=002`;
            console.error(msg);
            console.trace();
            return Q.reject({msg: msg});
        }

        let url = join(url_prefix, url_suffix);
        url += `?1=1`;
        url += `&private_token=${__gitlab_private_token}`;
        _.each(p, function (property_value, property_name) {
            url += `&${property_name}=${property_value}`;
        });
        return url;
    };

    const request = function (pParameter) {
        const defaultParameter = {};
        defaultParameter.url_suffix = ``;
        defaultParameter.gitlab_private_token = gitlab_private_token;
        defaultParameter.p = {};

        const parameter = extend({}, defaultParameter, pParameter);

        const url_suffix = parameter.url_suffix;
        const __gitlab_private_token = parameter.gitlab_private_token;
        const p = parameter.p;

        return Q.when()
            .then(function () {
                return get_request_url({
                    url_suffix: url_suffix,
                    gitlab_private_token: __gitlab_private_token,
                    p: p,
                });
            })
            .then(function (url) {
                return axios({
                    url,
                });
            })
            .then(function (response) {
                const data = response.data;
                return Q.resolve(data);
            })
            .catch(function (ex) {
                console.error(ex);
                console.trace();
                if (ex) {
                    if (ex.stack) {
                        console.error(ex.stack);
                    }
                }
                return Q.reject(ex);
            })
            ;
    };

    /**
     * 获取请求url
     * @method get_request_url
     *
     * @param {object} pParameter
     * @param {string} pParameter.url_suffix url后缀
     * @param {object} pParameter.p 参数对象，具体参考对应接口的官方文档
     *
     * @return {string} 请求url
     *
     * @demo src/system-docking/gitlab/client/demo/get_request_url.demo.js
     */
    __this.get_request_url = get_request_url;

    /**
     * 请求接口
     * @method request
     *
     * @param {object} pParameter
     * @param {string} pParameter.url_suffix url后缀
     * @param {object} pParameter.p 参数对象，具体参考对应接口的官方文档
     *
     * @return {object} promise
     *
     * @demo src/system-docking/gitlab/client/demo/request.demo.js
     */
    __this.request = request;
};

module.exports = Client;
