//<editor-fold desc="node core modules">
const path = require('path');
//</editor-fold>

//<editor-fold desc="npm modules">
const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
const _ = require('underscore');
const moment = require('moment');
const URI = require('urijs');
//</editor-fold>

const __exec_windows = require(`./__exec_windows.js`);

/**
 * NL类库
 *
 * @module NL
 */

/**
 * WinRar命令封装
 * <div>
 * <span class="tag margin-right">不支持浏览器环境</span>
 * <span class="tag margin-right">支持node环境</span>
 * </div>
 *
 * @class NL.systemDocking.winrar
 * @param {object} pInitParameter
 * @param {string} pInitParameter.path_winrar_exe winrar.exe的绝对路径，默认值为【C:\Program Files\WinRAR\WinRAR.exe】
 * @constructor
 * @return {object} NL.systemDocking.winrar实例
 *
 * @demo src/system-docking/winrar/demo/default.demo.js
 */
module.exports = function (pInitParameter) {
    const __this = this;

    const defaultInitParameter = {};
    defaultInitParameter.path_winrar_exe = `C:\\Program Files\\WinRAR\\WinRAR.exe`;

    const initParameter = extend(true, {}, defaultInitParameter, pInitParameter);

    const path_winrar_exe = initParameter.path_winrar_exe;

    const __lastInfo = {};
    __lastInfo.zip = null;
    __lastInfo.unzip = null;

    /**
     * 获取最后调用的相关信息
     *
     * @method getLastInfo
     * @return {object} 最后调用的相关信息
     *
     */
    const getLastInfo = function () {
        return __lastInfo;
    };

    /**
     * 创建zip文件
     *
     * @method zip
     * @param {object} pParameter
     * @param {string} pParameter.type 类型（000|001），默认为000
     * <br>type=000时，
     *
     * @param {string} pParameter.parameter['000'].path_dir_src 要创建zip文件的源目录，仅type=000时有效
     * @param {string} pParameter.parameter['000'].path_dir_dest zip创建的所在目录，仅type=000时有效
     * @param {function} pParameter.parameter['000'].get_file_name 获取zip文件名称，仅type=000时有效，非必需参数
     *
     * @param {string} pParameter.parameter['001'].path_dir_src 要创建zip文件的源目录，仅type=000时有效
     * @param {string} pParameter.parameter['001'].path_dir_dest zip创建的所在目录，仅type=000时有效
     * @param {function} pParameter.parameter['001'].get_file_name 获取zip文件名称，仅type=000时有效，非必需参数
     *
     * @return {object} zip文件创建结束的promise
     *
     * <br>promise.then(data => {});
     * <br>data.path_dir_src = ``;
     * <br>data.path_dir_dest = ``;
     * <br>data.file_name_dest = ``;
     * <br>data.path_file_dest = ``;
     * <br>示例
     * <br>path_dir_src="D:\\Dev\\ningkyolei\\nllib\\test-node\\system-docking\\winrar\\res\\zip\\000\\ning"
     * <br>path_dir_dest="D:\\Dev\\ningkyolei\\nllib\\test-node-build\\system-docking\\winrar\\zip\\000"
     * <br>file_name_dest="ning-20181015152856.zip"
     * <br>path_file_dest="D:\\Dev\\ningkyolei\\nllib\\test-node-build\\system-docking\\winrar\\zip\\000\\ning-20181015152856.zip"
     * <br>
     * <br>type=000与type=001的区别如下图所示
     * <img src="https://note.youdao.com/yws/api/personal/file/8A38D02392C34105AE3FC9FC78AF4CE9?method=download&shareKey=20b77f0a74e1234a52055a54aa84b028">
     * 
     */
    const zip = function (pParameter) {
        if (!pParameter) {
            pParameter = {};
        }

        if (!pParameter.parameter) {
            pParameter.parameter = {};
        }

        if (!pParameter.parameter['000']) {
            pParameter.parameter['000'] = {};
        }

        const type = pParameter.type || `000`;

        const now = moment();
        const now_format = now.format(`YYYYMMDDHHmmss`);

        const process_type_000 = function () {
            const path_dir_src = pParameter.parameter['000'].path_dir_src;
            const path_dir_dest = pParameter.parameter['000'].path_dir_dest;
            const _get_file_name = pParameter.parameter['000'].get_file_name;
            let get_file_name = _get_file_name;
            if (!_.isFunction(get_file_name)) {
                get_file_name = function (args) {
                    if (!args) {
                        args = {};
                    }

                    const path_dir_src = args.path_dir_src;
                    const path_dir_dest = args.path_dir_dest;
                    const now_format = args.now_format;

                    const basename = path.basename(path_dir_src);
                    const file_name_dest = `${basename}-${now_format}.zip`;
                    return file_name_dest;
                };
            }

            if (!fs.existsSync(path_dir_dest)) {
                console.log(`目录不存在，进行创建 ===> ${path_dir_dest}`);
                fs.ensureDirSync(path_dir_dest);
                console.log(`目录创建成功 ===> ${path_dir_dest}`);
            }

            const file_name_dest = get_file_name({
                path_dir_src: path_dir_src,
                path_dir_dest: path_dir_dest,
                now_format: now_format,
            });
            const path_file_dest = path.join(path_dir_dest, file_name_dest);

            const cmd = `"${path_winrar_exe}" a -ep1 "${path_file_dest}" "${path_dir_src}"`;
            console.log(cmd);
            const promise = __exec_windows(cmd).then(function () {
                console.log(`文件创建成功：${path_file_dest}`);

                const d = {
                    path_dir_src: path_dir_src,
                    path_dir_dest: path_dir_dest,
                    file_name_dest: file_name_dest,
                    path_file_dest: path_file_dest,
                };
                __lastInfo.zip = d;

                return Q.resolve(d);
            });
            return promise;
        };

        const process_type_001 = function () {
            const path_dir_src = pParameter.parameter['001'].path_dir_src;
            const path_dir_dest = pParameter.parameter['001'].path_dir_dest;
            const _get_file_name = pParameter.parameter['001'].get_file_name;
            let get_file_name = _get_file_name;
            if (!_.isFunction(get_file_name)) {
                get_file_name = function (args) {
                    if (!args) {
                        args = {};
                    }

                    const path_dir_src = args.path_dir_src;
                    const path_dir_dest = args.path_dir_dest;
                    const now_format = args.now_format;

                    const basename = path.basename(path_dir_src);
                    const file_name_dest = `${basename}-${now_format}.zip`;
                    return file_name_dest;
                };
            }

            if (!fs.existsSync(path_dir_dest)) {
                console.log(`目录不存在，进行创建 ===> ${path_dir_dest}`);
                fs.ensureDirSync(path_dir_dest);
                console.log(`目录创建成功 ===> ${path_dir_dest}`);
            }

            const file_name_dest = get_file_name({
                path_dir_src: path_dir_src,
                path_dir_dest: path_dir_dest,
                now_format: now_format,
            });
            const path_file_dest = path.join(path_dir_dest, file_name_dest);

            let cmd = ``;
            cmd += `cd /D "${path_dir_src}"`;
            cmd += ` & `;
            cmd += `"${path_winrar_exe}" a -r "${path_file_dest}" "*.*"`;
            console.log(cmd);
            const promise = __exec_windows(cmd).then(function () {
                console.log(`文件创建成功：${path_file_dest}`);

                const d = {
                    path_dir_src: path_dir_src,
                    path_dir_dest: path_dir_dest,
                    file_name_dest: file_name_dest,
                    path_file_dest: path_file_dest,
                };
                __lastInfo.zip = d;

                return Q.resolve(d);
            });
            return promise;
        };

        let r;
        switch (type) {
            case `000`:
                r = process_type_000();
                break;

            case `001`:
                r = process_type_001();
                break;
        }
        return r;
    };

    /**
     * 解压zip文件
     *
     * @method unzip
     * @param {object} pParameter
     * @param {string} pParameter.type 类型（000），默认为000
     * @param {string} pParameter.parameter['000'].path_zip_src 要解压zip文件的路径，仅type=000时有效
     * @param {string} pParameter.parameter['000'].path_dir_dest zip解压到的目录，仅type=000时有效
     * @return {object} zip文件解压结束的promise
     * <br>promise.then(data => {});
     * <br>type=000时
     * <br>data.path_zip_src = ``;
     * <br>data.path_dir_dest = ``;
     * <br>示例
     * <br>path_zip_src="Z:\\Dev\\ningkyolei\\nllib\\test\\cli\\NL.Plugins.WinRar\\res\\000\\dir-xxx.zip"
     * <br>path_dir_dest="Z:\\Dev\\ningkyolei\\nllib\\test\\cli\\NL.Plugins.WinRar\\temp\\000"
     *
     */
    const unzip = function (pParameter) {
        if (!pParameter) {
            pParameter = {};
        }

        if (!pParameter.parameter) {
            pParameter.parameter = {};
        }

        if (!pParameter.parameter['000']) {
            pParameter.parameter['000'] = {};
        }

        const type = pParameter.type || `000`;

        const process_type_000 = function () {
            const path_zip_src = pParameter.parameter['000'].path_zip_src;
            const path_dir_dest = pParameter.parameter['000'].path_dir_dest;

            if (!fs.existsSync(path_dir_dest)) {
                console.log(`目录不存在，进行创建 ===> ${path_dir_dest}`);
                fs.ensureDirSync(path_dir_dest);
                console.log(`目录创建成功 ===> ${path_dir_dest}`);
            }

            const cmd = `"${path_winrar_exe}" x -o+ "${path_zip_src}" "${path_dir_dest}"`;
            console.log(cmd);
            const promise = __exec_windows(cmd).then(function () {
                console.log(`解压成功：【${path_zip_src}】 ===> 【${path_dir_dest}】`);

                const d = {
                    path_zip_src: path_zip_src,
                    path_dir_dest: path_dir_dest,
                };
                __lastInfo.unzip = d;

                return Q.resolve(d);
            });
            return promise;
        };

        let r;
        switch (type) {
            case `000`:
                r = process_type_000();
                break;
        }
        return r;
    };

    __this.getLastInfo = getLastInfo;
    __this.zip = zip;
    __this.unzip = unzip;
};